@@ -1,8 +1,10 @@
 import gc
 import json
 from dataclasses import dataclass
+from io import BytesIO
 from typing import Optional
 
+import requests
 import streamlit as st
 import torch
 from diffusers import StableDiffusionInpaintPipeline
@@ -18,6 +20,10 @@ from diffuzers import utils
 class Inpainting:
     model: Optional[str] = None
     device: Optional[str] = None
+    output_path: Optional[str] = None
+
+    def __str__(self) -> str:
+        return f"Inpainting(model={self.model}, device={self.device}, output_path={self.output_path})"
 
     def __post_init__(self):
         self.pipeline = StableDiffusionInpaintPipeline.from_pretrained(
@@ -31,6 +37,28 @@ class Inpainting:
         self.scheduler_config = self.pipeline.scheduler.config
         self.compatible_schedulers = {scheduler.__name__: scheduler for scheduler in self._compatible_schedulers}
 
+        if self.device == "mps":
+            self.pipeline.enable_attention_slicing()
+            # warmup
+
+            def download_image(url):
+                response = requests.get(url)
+                return Image.open(BytesIO(response.content)).convert("RGB")
+
+            img_url = "https://raw.githubusercontent.com/CompVis/latent-diffusion/main/data/inpainting_examples/overture-creations-5sI6fQgYIuo.png"
+            mask_url = "https://raw.githubusercontent.com/CompVis/latent-diffusion/main/data/inpainting_examples/overture-creations-5sI6fQgYIuo_mask.png"
+
+            init_image = download_image(img_url).resize((512, 512))
+            mask_image = download_image(mask_url).resize((512, 512))
+
+            prompt = "Face of a yellow cat, high resolution, sitting on a park bench"
+            _ = self.pipeline(
+                prompt=prompt,
+                image=init_image,
+                mask_image=mask_image,
+                num_inference_steps=2,
+            )
+
     def _set_scheduler(self, scheduler_name):
         scheduler = self.compatible_schedulers[scheduler_name].from_config(self.scheduler_config)
         self.pipeline.scheduler = scheduler
@@ -40,7 +68,12 @@ class Inpainting:
     ):
         self._set_scheduler(scheduler)
         logger.info(self.pipeline.scheduler)
-        generator = torch.Generator(device=self.device).manual_seed(seed)
+
+        if self.device == "mps":
+            generator = torch.manual_seed(seed)
+            num_images = 1
+        else:
+            generator = torch.Generator(device=self.device).manual_seed(seed)
 
         output_images = self.pipeline(
             prompt=prompt,
@@ -65,6 +98,14 @@ class Inpainting:
         metadata = json.dumps(metadata)
         _metadata = PngInfo()
         _metadata.add_text("inpainting", metadata)
+
+        utils.save_images(
+            images=output_images,
+            module="inpainting",
+            metadata=metadata,
+            output_path=self.output_path,
+        )
+
         torch.cuda.empty_cache()
         gc.collect()
         return output_images, _metadata
@@ -78,17 +119,26 @@ class Inpainting:
         with col2:
             negative_prompt = st.text_area("Negative Prompt", "")
 
-        uploaded_file = st.sidebar.file_uploader(
+        uploaded_file = st.file_uploader(
             "Image:", type=["png", "jpg", "jpeg"], help="Image size must match model's image size. Usually: 512 or 768"
         )
 
         # sidebar options
         available_schedulers = list(self.compatible_schedulers.keys())
+        if "EulerAncestralDiscreteScheduler" in available_schedulers:
+            available_schedulers.insert(
+                0, available_schedulers.pop(available_schedulers.index("EulerAncestralDiscreteScheduler"))
+            )
         scheduler = st.sidebar.selectbox("Scheduler", available_schedulers, index=0)
         guidance_scale = st.sidebar.slider("Guidance scale", 1.0, 40.0, 7.5, 0.5)
         num_images = st.sidebar.slider("Number of images per prompt", 1, 30, 1, 1)
         steps = st.sidebar.slider("Steps", 1, 150, 50, 1)
-        seed = st.sidebar.slider("Seed", 1, 999999, 1, 1)
+        seed_placeholder = st.sidebar.empty()
+        seed = seed_placeholder.number_input("Seed", value=42, min_value=1, max_value=999999, step=1)
+        random_seed = st.sidebar.button("Random seed")
+        _seed = torch.randint(1, 999999, (1,)).item()
+        if random_seed:
+            seed = seed_placeholder.number_input("Seed", value=_seed, min_value=1, max_value=999999, step=1)
 
         if uploaded_file is not None:
             drawing_mode = st.selectbox("Drawing tool:", ("freedraw", "rect", "circle"))
@@ -102,9 +152,9 @@ class Inpainting:
                 background_color=bg_color,
                 background_image=pil_image,
                 update_streamlit=True,
+                drawing_mode=drawing_mode,
                 height=768,
                 width=768,
-                drawing_mode=drawing_mode,
                 key="canvas",
             )
             submit = st.button("Generate")
@@ -118,7 +168,7 @@ class Inpainting:
                 # convert mask npy to PIL image
                 mask_pil = Image.fromarray(mask_npy).convert("RGB")
                 # resize mask to match image size
-                mask_pil = mask_pil.resize((img_width, img_height), resample=Image.Resampling.LANCZOS)
+                mask_pil = mask_pil.resize((img_width, img_height), resample=Image.LANCZOS)
                 with st.spinner("Generating..."):
                     output_images, metadata = self.generate_image(
                         prompt=prompt,
